import { find } from '@lumino/algorithm';
import { UUID } from '@lumino/coreutils';
import { SessionManager, KernelSpecManager } from '@jupyterlab/services';
import { getCookie } from '../../utils/Utils';
const JUPYTER_REACT_PATH_COOKIE_NAME = "jupyter-react-kernel-path";
export class Kernel {
    _clientId;
    _connectionStatus;
    _id;
    _info;
    _kernelConnection;
    _kernelManager;
    _kernelName;
    _kernelSpecManager;
    _kernelSpecName;
    _kernelType;
    _path;
    _ready;
    _readyResolve;
    _serverSettings;
    _session;
    _sessionId;
    _sessionManager;
    constructor(props) {
        const { kernelManager, kernelName, kernelType, kernelSpecName, kernelModel, serverSettings } = props;
        this._kernelSpecManager = new KernelSpecManager({ serverSettings });
        this._kernelManager = kernelManager;
        this._kernelName = kernelName;
        this._kernelType = kernelType;
        this._kernelSpecName = kernelSpecName;
        this._serverSettings = serverSettings;
        this.initReady = this.initReady.bind(this);
        this.initReady();
        this.requestKernel(kernelModel);
    }
    initReady() {
        this._ready = new Promise((resolve, _) => {
            this._readyResolve = resolve;
        });
    }
    async requestKernel(kernelModel) {
        await this._kernelManager.ready;
        this._sessionManager = new SessionManager({
            kernelManager: this._kernelManager,
            serverSettings: this._serverSettings,
            standby: 'never',
        });
        await this._sessionManager.ready;
        if (kernelModel) {
            console.log('Reusing a pre-existing kernel model.');
            await this._sessionManager.refreshRunning();
            const model = find(this._sessionManager.running(), (item) => {
                return item.path === this._path;
            });
            if (model) {
                this._session = this._sessionManager.connectTo({ model });
            }
        }
        else {
            let path = getCookie(this.cookieName);
            if (!path) {
                path = "kernel-" + UUID.uuid4();
                document.cookie = this.cookieName + "=" + path;
            }
            this._path = path;
            this._session = await this._sessionManager.startNew({
                name: this._kernelName,
                path: this._path,
                type: this._kernelType,
                kernel: {
                    name: this._kernelSpecName,
                },
            }, {
                kernelConnectionOptions: {
                    handleComms: true,
                }
            });
        }
        this._kernelConnection = this._session.kernel;
        const updateConnectionStatus = () => {
            if (this._connectionStatus === 'connected') {
                this._clientId = this._session.kernel.clientId;
                this._id = this._session.kernel.id;
                this._readyResolve();
            }
        };
        if (this._kernelConnection) {
            this._sessionId = this._session.id;
            this._connectionStatus = this._kernelConnection.connectionStatus;
            updateConnectionStatus();
            this._kernelConnection.connectionStatusChanged.connect((_, connectionStatus) => {
                //        this.initReady();
                this._connectionStatus = connectionStatus;
                updateConnectionStatus();
            });
            this._kernelConnection.info.then((info) => {
                this._info = info;
                console.log(`The Kernel is ready`, this.toJSON());
            });
        }
    }
    get cookieName() {
        return JUPYTER_REACT_PATH_COOKIE_NAME + "_" + this._kernelSpecName;
    }
    get ready() {
        return this._ready;
    }
    get clientId() {
        return this._clientId;
    }
    get id() {
        return this._id;
    }
    get sessionId() {
        return this._sessionId;
    }
    get info() {
        return this._info;
    }
    get session() {
        return this._session;
    }
    get kernelManager() {
        return this._kernelManager;
    }
    get kernelSpecManager() {
        return this._kernelSpecManager;
    }
    get sessionManager() {
        return this._sessionManager;
    }
    get path() {
        return this._path;
    }
    get connection() {
        return this._kernelConnection;
    }
    toJSON() {
        return {
            path: this._path,
            id: this.id,
            clientId: this.clientId,
            sessionId: this.sessionId,
            kernelInfo: this.info,
        };
    }
    toString() {
        return `id:${this.id} - client_id:${this.clientId} - session_id:${this.sessionId} - path:${this._path}`;
    }
    shutdown() {
        this._session.kernel?.shutdown();
        this.connection?.dispose();
    }
}
export default Kernel;
//# sourceMappingURL=Kernel.js.map